package eu.beesoft.gantt.demo;

import java.util.Date;
import eu.beesoft.gaia.swing.TreeTableNode;
import eu.beesoft.gantt.GanttModel;
import eu.beesoft.gantt.TimelineObject;
import eu.beesoft.gantt.treetable.GanttNode;
import eu.beesoft.gantt.undo.UndoStep;

@SuppressWarnings ("serial")
public class Model extends GanttModel {

	public Model () {
		super ();
	}



	@Override
	public Class<?> getColumnClass (int column) {
		switch (column) {
			case 2:
			case 3:
				return Date.class;
			default:
				return super.getColumnClass (column);
		}
	}



	@Override
	public boolean isCellEditable (TreeTableNode node, int column) {
		if (column == 0) {
			return true;
		}
		return (column >= 2);
	}



	@Override
	public void explore (GanttNode node) {
		Task task = (Task) node.getUserObject ();
		for (Task t : task.getSubtask ()) {
			GanttNode child = new GanttNode (this, t);
			if (t.getSubtask ().isEmpty ()) {
				child.setExplorable (false);
			}
			node.add (child);
		}
	}



	@Override
	public boolean moveObject (TimelineObject object,
	        TimelineObject newContainer, int newIndex, UndoStep undo) {
		Task task = (Task) object;
		Task supertask = (Task) newContainer;
		undo.registerObject (task);
		undo.registerObject (supertask);
		Task oldSupertask = task.getSupertask ();
		if (supertask != oldSupertask) {
			if (oldSupertask != null) {
				undo.registerObject (oldSupertask);
				oldSupertask.removeSubtask (task);
			}
			supertask.addSubtask (task);
		}
		supertask.shiftSubtask (task, newIndex);
		return true;
	}



	@Override
	public boolean deleteObject (TimelineObject what, UndoStep undo) {
		Task task = (Task) what;
		Task parent = task.getSupertask ();
		undo.registerObject (parent);
		undo.registerObject (task);
		parent.removeSubtask (task);
		return true;
	}



	@Override
	public TimelineObject createObject (TimelineObject parent, int index,
	        UndoStep undo) {
		undo.registerObject (parent);
		Task parentTask = (Task) parent;
		Task task = new Task ();
		String id = DemoGantt.getNextTaskId ();
		task.setTaskId (id);
		task.setName ("Task " + id);
		parentTask.addSubtask (task);
		parentTask.shiftSubtask (task, index);
		return task;
	}



	@Override
	public String getLabel (TimelineObject object) {
		if (object instanceof Task) {
			Resource assignee = ((Task) object).getAssignee ();
			if (assignee != null) {
				return assignee.getName ();
			}
		}
		return null;
	}
}
